"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MariaDBConnector = void 0;
const fs = __importStar(require("fs-extra"));
const knex_1 = __importDefault(require("knex"));
const log4js_1 = require("log4js");
const path = __importStar(require("path"));
const URI = __importStar(require("uri-js"));
const generators_1 = require("../generation/generators/generators");
const schema_class_1 = require("../schema/schema.class");
class MariaDBConnector {
    constructor(uri) {
        this.uri = uri;
        this.triggers = [];
        this.logger = log4js_1.getLogger();
        this.triggerBackupFile = path.join('settings', 'triggers.json');
        this.extractForeignKeys = (table) => __awaiter(this, void 0, void 0, function* () {
            const foreignKeys = yield this.getForeignKeys(table);
            table.referencedTables = [];
            for (const column of table.columns) {
                const match = foreignKeys.find((fk) => fk.column.toLowerCase() === column.name.toLowerCase());
                if (match) {
                    column.generator = generators_1.Generators.foreignKey;
                    column.foreignKey = { table: match.foreignTable, column: match.foreignColumn };
                    column.unique = column.unique || match.uniqueIndex || false;
                    table.referencedTables.push(column.foreignKey.table);
                }
            }
        });
        this.uriComponents = URI.parse(this.uri);
        if (!this.uriComponents.path)
            throw new Error('Please sepcify database name');
        this.database = this.uriComponents.path.replace('/', '');
        this.dbConnection = knex_1.default({
            client: 'mysql',
            connection: this.uri,
            log: {
                warn: (message) => {
                    this.logger.warn(message);
                },
                error: (message) => {
                    this.logger.error(message);
                },
                deprecate: (message) => {
                    this.logger.warn(message);
                },
                debug: (message) => {
                    this.logger.debug(message);
                },
            },
            pool: {
                afterCreate: (conn, done) => {
                    conn.query('SET foreign_key_checks = OFF;', (err1) => {
                        if (err1)
                            done(err1, conn);
                        else
                            conn.query('SET autocommit = OFF;', (err2) => {
                                if (err2)
                                    done(err2, conn);
                                else
                                    conn.query('SET unique_checks = OFF;', (err3) => {
                                        done(err3, conn);
                                    });
                            });
                    });
                },
            },
        }).on('query-error', (err) => {
            this.logger.error(err.code, err.name);
        });
        if (fs.existsSync(this.triggerBackupFile)) {
            this.triggers = fs.readJSONSync(this.triggerBackupFile);
        }
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.warn(`For performance foreign_key_checks, autocommit and unique_checks are disabled during insert.`);
            this.logger.warn(`They are disabled per connections and should not alter your configuration.`);
            this.logger.info(`To improve performances further you can update innodb_autoinc_lock_mode = 0 in your my.ini.`);
        });
    }
    countLines(table) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.dbConnection(table.name).count())[0]['count(*)'];
        });
    }
    emptyTable(table) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.dbConnection.raw(`DELETE FROM ${table.name}`);
            yield this.dbConnection.raw(`ALTER TABLE ${table.name} AUTO_INCREMENT = 1;`);
        });
    }
    executeRawQuery(query) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.dbConnection.raw(query);
        });
    }
    insert(table, rows) {
        return __awaiter(this, void 0, void 0, function* () {
            if (rows.length === 0)
                return 0;
            const query = this.dbConnection(table)
                .insert(rows)
                .toQuery()
                .replace('insert into', 'insert ignore into');
            const insertResult = yield this.dbConnection.raw(query);
            yield this.dbConnection.raw('COMMIT;');
            return insertResult[0].affectedRows;
        });
    }
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.dbConnection.destroy();
        });
    }
    getSchema() {
        return __awaiter(this, void 0, void 0, function* () {
            let tables = yield this.getTablesInformation();
            tables = yield Promise.all(tables.map((table) => __awaiter(this, void 0, void 0, function* () {
                yield this.extractColumns(table);
                yield this.extractForeignKeys(table);
                return table;
            })));
            return schema_class_1.Schema.fromJSON({ tables });
        });
    }
    extractColumns(table) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info(table.name);
            const columns = yield this.getColumnsInformation(table);
            columns
                .filter((column) => {
                return ['enum', 'set'].includes(column.DATA_TYPE || '');
            }).forEach((column) => {
                column.NUMERIC_PRECISION = column.COLUMN_TYPE.match(/(enum|set)\((.*)\)$/)[1].split('\',\'').length;
            });
            table.columns = columns.map((mysqlColumn) => {
                const column = new schema_class_1.Column();
                column.name = mysqlColumn.COLUMN_NAME;
                if (mysqlColumn.COLUMN_KEY && mysqlColumn.COLUMN_KEY.match(/PRI|UNI/ig))
                    column.unique = true;
                column.nullable = mysqlColumn.IS_NULLABLE === 'YES' ? 0.1 : 0;
                column.max = mysqlColumn.CHARACTER_MAXIMUM_LENGTH || mysqlColumn.NUMERIC_PRECISION || 255;
                if (mysqlColumn.COLUMN_TYPE && mysqlColumn.COLUMN_TYPE.includes('unsigned'))
                    column.unsigned = true;
                if (mysqlColumn.EXTRA && mysqlColumn.EXTRA.includes('auto_increment'))
                    column.autoIncrement = true;
                switch (mysqlColumn.DATA_TYPE) {
                    case 'bool':
                    case 'boolean':
                        column.generator = generators_1.Generators.boolean;
                        break;
                    case 'smallint':
                        column.generator = generators_1.Generators.integer;
                        if (column.unsigned) {
                            column.min = 0;
                            column.max = 65535;
                        }
                        else {
                            column.min = -32768;
                            column.max = 32767;
                        }
                        break;
                    case 'mediumint':
                        column.generator = generators_1.Generators.integer;
                        if (column.unsigned) {
                            column.min = 0;
                            column.max = 16777215;
                        }
                        else {
                            column.min = -8388608;
                            column.max = 8388607;
                        }
                        break;
                    case 'tinyint':
                        column.generator = generators_1.Generators.integer;
                        if (column.unsigned) {
                            column.min = 0;
                            column.max = 255;
                        }
                        else {
                            column.min = -128;
                            column.max = 127;
                        }
                        break;
                    case 'int':
                    case 'integer':
                    case 'bigint':
                        column.generator = generators_1.Generators.integer;
                        if (column.unsigned) {
                            column.min = 0;
                            column.max = 2147483647;
                        }
                        else {
                            column.min = -2147483648;
                            column.max = 2147483647;
                        }
                        break;
                    case 'decimal':
                    case 'dec':
                    case 'float':
                    case 'double':
                        column.generator = generators_1.Generators.real;
                        if (column.unsigned) {
                            column.min = 0;
                            column.max = 2147483647;
                        }
                        else {
                            column.min = -2147483648;
                            column.max = 2147483647;
                        }
                        break;
                    case 'date':
                    case 'datetime':
                    case 'timestamp':
                        column.generator = generators_1.Generators.date;
                        column.minDate = '01-01-1970';
                        column.maxDate = undefined;
                        break;
                    case 'time':
                        column.generator = generators_1.Generators.time;
                        break;
                    case 'year':
                        column.generator = generators_1.Generators.integer;
                        column.min = 1901;
                        column.max = 2155;
                        break;
                    case 'varchar':
                    case 'char':
                    case 'binary':
                    case 'varbinary':
                    case 'tinyblob':
                    case 'text':
                    case 'tinytext':
                    case 'mediumtext':
                    case 'longtext':
                    case 'blob':
                    case 'mediumblob': // 16777215
                    case 'longblob': // 4,294,967,295
                        column.generator = generators_1.Generators.string;
                        break;
                    case 'bit':
                    case 'set':
                        column.generator = generators_1.Generators.bit;
                        column.max = mysqlColumn.NUMERIC_PRECISION;
                        break;
                    case 'enum':
                        column.generator = generators_1.Generators.integer;
                        column.max = mysqlColumn.NUMERIC_PRECISION;
                        break;
                }
                return column;
            });
        });
    }
    backupTriggers(tables) {
        return __awaiter(this, void 0, void 0, function* () {
            const triggers = yield this.dbConnection
                .select()
                .from('information_schema.TRIGGERS')
                .where('event_object_schema', this.database)
                .whereIn(`event_object_table`, tables);
            this.triggers = this.triggers.concat(triggers);
            fs.writeJSONSync(this.triggerBackupFile, this.triggers);
        });
    }
    cleanBackupTriggers() {
        fs.unlinkSync(this.triggerBackupFile);
    }
    disableTriggers(table) {
        return __awaiter(this, void 0, void 0, function* () {
            const triggers = this.triggers.filter((trigger) => {
                return trigger.EVENT_OBJECT_SCHEMA === this.database && trigger.EVENT_OBJECT_TABLE === table;
            });
            const promises = triggers.map((trigger) => {
                return this.dbConnection.raw(`DROP TRIGGER IF EXISTS ${trigger.TRIGGER_SCHEMA}.${trigger.TRIGGER_NAME};`);
            });
            yield Promise.all(promises)
                .catch(err => this.logger.error(err.message));
        });
    }
    enableTriggers(table) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let i = 0; i < this.triggers.length; i++) {
                const trigger = this.triggers[i];
                if (trigger.EVENT_OBJECT_SCHEMA !== this.database || trigger.EVENT_OBJECT_TABLE !== table)
                    continue;
                yield this.dbConnection.raw(`DROP TRIGGER IF EXISTS ${trigger.TRIGGER_SCHEMA}.${trigger.TRIGGER_NAME};`);
                yield this.dbConnection.raw(`CREATE DEFINER = ${trigger.DEFINER}
                TRIGGER ${trigger.TRIGGER_SCHEMA}.${trigger.TRIGGER_NAME} ${trigger.ACTION_TIMING} ${trigger.EVENT_MANIPULATION}
                ON ${trigger.EVENT_OBJECT_SCHEMA}.${trigger.EVENT_OBJECT_TABLE}
                FOR EACH ROW
                ${trigger.ACTION_STATEMENT}`);
                this.triggers.splice(i, 1);
            }
        });
    }
    getTablesInformation() {
        return __awaiter(this, void 0, void 0, function* () {
            const tableNames = yield this.dbConnection
                .select([
                this.dbConnection.raw('t.TABLE_NAME AS name'),
            ])
                .from('information_schema.tables as t')
                .where('t.TABLE_SCHEMA', this.database)
                .andWhere('t.TABLE_TYPE', 'BASE TABLE')
                .groupBy('t.TABLE_SCHEMA', 't.TABLE_NAME');
            return tableNames.map((row) => {
                const table = new schema_class_1.Table();
                table.name = row.name;
                return table;
            });
        });
    }
    getColumnsInformation(table) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.dbConnection.select()
                .from('information_schema.COLUMNS')
                .where({
                'TABLE_SCHEMA': this.database,
                'TABLE_NAME': table.name,
            });
        });
    }
    getForeignKeys(table) {
        return __awaiter(this, void 0, void 0, function* () {
            const subQuery = this.dbConnection
                .select([
                'kcu2.table_name',
                'kcu2.column_name',
                'kcu2.constraint_schema',
                this.dbConnection.raw('1 AS unique_index'),
            ])
                .from('information_schema.KEY_COLUMN_USAGE AS kcu2')
                .innerJoin('information_schema.TABLE_CONSTRAINTS AS tc', function () {
                this.on('tc.CONSTRAINT_SCHEMA', '=', 'kcu2.CONSTRAINT_SCHEMA')
                    .andOn('tc.TABLE_NAME', '=', 'kcu2.TABLE_NAME')
                    .andOn('tc.CONSTRAINT_NAME', '=', 'kcu2.CONSTRAINT_NAME')
                    .andOnIn('tc.CONSTRAINT_TYPE', ['PRIMARY KEY', 'UNIQUE']);
            })
                .groupBy(['kcu2.TABLE_NAME', 'kcu2.CONSTRAINT_NAME'])
                .having(this.dbConnection.raw('count(kcu2.CONSTRAINT_NAME) < 2'))
                .as('indexes');
            return this.dbConnection.select([
                'kcu.column_name AS column',
                'kcu.referenced_table_name AS foreignTable',
                'kcu.referenced_column_name AS foreignColumn',
                'unique_index AS uniqueIndex',
            ])
                .from('information_schema.key_column_usage as kcu')
                .leftJoin(subQuery, function () {
                this.on('kcu.table_name', 'indexes.table_name')
                    .andOn('kcu.column_name', 'indexes.column_name')
                    .andOn('kcu.constraint_schema', 'indexes.constraint_schema');
            })
                .where('kcu.table_name', table.name)
                .whereNotNull('kcu.referenced_column_name');
        });
    }
    getValuesForForeignKeys(table, column, foreignTable, foreignColumn, limit, unique, condition) {
        return __awaiter(this, void 0, void 0, function* () {
            let values = [];
            const query = this.dbConnection(foreignTable)
                .distinct(`${foreignTable}.${foreignColumn}`)
                .limit(limit);
            if (condition) {
                query.andWhere(this.dbConnection.raw(condition));
            }
            if (unique) {
                query.leftJoin(table, function () {
                    this.on(`${table}.${column}`, `${foreignTable}.${foreignColumn}`);
                }).whereNull(`${table}.${column}`);
            }
            values = (yield query).map(result => result[foreignColumn]);
            return values;
        });
    }
}
exports.MariaDBConnector = MariaDBConnector;
