"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DateGenerator = void 0;
const generators_1 = require("./generators");
const schema_class_1 = require("../../schema/schema.class");
class DateGenerator extends generators_1.AbstractGenerator {
    constructor() {
        super(...arguments);
        this.values = [];
    }
    validate() {
        if (this.column.minDate === undefined)
            throw new Error(`minDate value required for type date: ${this.table.name}.${this.column.name}`);
        if (this.column.maxDate === undefined)
            throw new Error(`maxDate value required for type date: ${this.table.name}.${this.column.name}`);
        return true;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.column.monotonic !== undefined && this.column.monotonic !== schema_class_1.Monotonic.NONE) {
                this.monotonic(this.column.monotonic);
            }
            return this;
        });
    }
    monotonic(order) {
        const dates = new Array(this.table.deltaRows)
            .fill(true)
            .map(() => this.generateRandomDate());
        if (order === schema_class_1.Monotonic.ASC) {
            dates.sort((a, b) => a.getTime() - b.getTime());
        }
        else {
            dates.sort((a, b) => b.getTime() - a.getTime());
        }
        this.values = dates;
    }
    generateRandomDate() {
        const min = this.column.minDate ? new Date(this.column.minDate) : new Date('01-01-1970');
        const max = this.column.maxDate ? new Date(this.column.maxDate) : new Date();
        return this.random.date(min, max);
    }
    generate(rowIndex, row) {
        let result;
        if (this.values[rowIndex])
            result = this.values[rowIndex];
        else
            result = this.generateRandomDate();
        return result;
    }
}
exports.DateGenerator = DateGenerator;
