"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ForeignKeyGenerator = void 0;
const generators_1 = require("./generators");
class ForeignKeyGenerator extends generators_1.AbstractGenerator {
    constructor() {
        super(...arguments);
        this.values = [];
    }
    validate() {
        if (this.column.foreignKey === undefined)
            throw new Error(`foreignKey object required for type foreignKey: ${this.table.name}.${this.column.name}`);
        if (this.column.foreignKey.table === undefined)
            throw new Error(`foreignKey.table value required for type foreignKey: ${this.table.name}.${this.column.name}`);
        if (this.column.foreignKey.column === undefined)
            throw new Error(`foreignKey.table value required for type foreignKey: ${this.table.name}.${this.column.name}`);
        return true;
    }
    setDbConnector(dbConnector) {
        this.dbConnector = dbConnector;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.dbConnector)
                throw new Error('Could not connect to database');
            const values = yield this.dbConnector.getValuesForForeignKeys(this.table.name, this.column.name, this.column.foreignKey.table, this.column.foreignKey.column, this.table.deltaRows, this.column.unique, this.column.foreignKey.where);
            this.values = this.random.shuffle(values);
            return this;
        });
    }
    generate(rowIndex, row) {
        const foreignKeys = this.values;
        if (rowIndex < foreignKeys.length)
            return foreignKeys[rowIndex];
        else if (!this.column.unique) {
            return foreignKeys[rowIndex % foreignKeys.length];
        }
        else {
            if (this.column.nullable <= 0) {
                throw new Error(`Not enough FK for column: ${this.table.name}.${this.column.name}`);
            }
        }
    }
}
exports.ForeignKeyGenerator = ForeignKeyGenerator;
