"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GeneratorBuilder = void 0;
const bit_generator_1 = require("./bit.generator");
const boolean_generator_1 = require("./boolean.generator");
const date_generator_1 = require("./date.generator");
const foreignkey_generator_1 = require("./foreignkey.generator");
const function_generator_1 = require("./function.generator");
const generators_1 = require("./generators");
const integer_generator_1 = require("./integer.generator");
const real_generator_1 = require("./real.generator");
const string_generator_1 = require("./string.generator");
const time_generator_1 = require("./time.generator");
const values_generator_1 = require("./values.generator");
class GeneratorBuilder {
    constructor(random, dbConnector, table) {
        this.random = random;
        this.dbConnector = dbConnector;
        this.table = table;
    }
    static validate(table, column) {
        switch (column.generator) {
            case generators_1.Generators.bit:
                bit_generator_1.BitGenerator.validate(table, column);
                break;
            case generators_1.Generators.boolean:
                boolean_generator_1.BooleanGenerator.validate(table, column);
                break;
            case generators_1.Generators.integer:
                integer_generator_1.IntegerGenerator.validate(table, column);
                break;
            case generators_1.Generators.real:
                real_generator_1.RealGenerator.validate(table, column);
                break;
            case generators_1.Generators.date:
                date_generator_1.DateGenerator.validate(table, column);
                break;
            case generators_1.Generators.time:
                time_generator_1.TimeGenerator.validate(table, column);
                break;
            case generators_1.Generators.string:
                string_generator_1.StringGenerator.validate(table, column);
                break;
            case generators_1.Generators.values:
                values_generator_1.ValuesGenerator.validate(table, column);
                break;
            case generators_1.Generators.foreignKey:
                foreignkey_generator_1.ForeignKeyGenerator.validate(table, column);
                break;
            case generators_1.Generators.function:
                function_generator_1.FunctionGenerator.validate(table, column);
                break;
            case generators_1.Generators.none:
            default:
                throw new Error(`No generator defined for column: ${table.name}.${column.name}`);
        }
        return true;
    }
    build(column) {
        let generator;
        switch (column.generator) {
            case generators_1.Generators.bit:
                generator = new bit_generator_1.BitGenerator(this.random, this.table, column);
                break;
            case generators_1.Generators.boolean:
                generator = new boolean_generator_1.BooleanGenerator(this.random, this.table, column);
                break;
            case generators_1.Generators.integer:
                generator = new integer_generator_1.IntegerGenerator(this.random, this.table, column);
                break;
            case generators_1.Generators.real:
                generator = new real_generator_1.RealGenerator(this.random, this.table, column);
                break;
            case generators_1.Generators.date:
                generator = new date_generator_1.DateGenerator(this.random, this.table, column);
                break;
            case generators_1.Generators.time:
                generator = new time_generator_1.TimeGenerator(this.random, this.table, column);
                break;
            case generators_1.Generators.string:
                generator = new string_generator_1.StringGenerator(this.random, this.table, column);
                break;
            case generators_1.Generators.values:
                generator = new values_generator_1.ValuesGenerator(this.random, this.table, column);
                break;
            case generators_1.Generators.foreignKey:
                generator = new foreignkey_generator_1.ForeignKeyGenerator(this.random, this.table, column);
                generator.setDbConnector(this.dbConnector);
                break;
            case generators_1.Generators.function:
                generator = new function_generator_1.FunctionGenerator(this.random, this.table, column);
                break;
            case generators_1.Generators.none:
            default:
                throw new Error(`No generator defined for column: ${this.table.name}.${column.name}`);
        }
        return generator;
    }
}
exports.GeneratorBuilder = GeneratorBuilder;
