"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomTable = exports.CustomSchema = exports.CustomSettings = void 0;
const class_transformer_1 = require("class-transformer");
const class_validator_1 = require("class-validator");
const database_engines_1 = require("../database/database-engines");
class CustomSettings {
    constructor() {
        this.beforeAll = [];
        this.afterAll = [];
        this.engine = database_engines_1.DatabaseEngines.MARIADB;
        this.disableTriggers = false;
        this.ignoredTables = [];
        this.tablesToFill = [];
        /**
         * A default value has been provided to maxLengthValue as this can drastically improve performances. This is a parameter which used to be
         * defined a lot by user anyway. The value of 36 has been chosen to allow usage of UID in string generator.
         */
        this.maxLengthValue = 36;
        this.values = {};
        this.options = [];
        this.maxRowsPerBatch = 1000;
    }
}
__decorate([
    class_validator_1.IsArray(),
    class_validator_1.ValidateNested({ each: true }),
    __metadata("design:type", Array)
], CustomSettings.prototype, "beforeAll", void 0);
__decorate([
    class_validator_1.IsArray(),
    class_validator_1.ValidateNested({ each: true }),
    __metadata("design:type", Array)
], CustomSettings.prototype, "afterAll", void 0);
__decorate([
    class_validator_1.IsEnum(database_engines_1.DatabaseEngines),
    __metadata("design:type", String)
], CustomSettings.prototype, "engine", void 0);
__decorate([
    class_validator_1.IsBoolean(),
    __metadata("design:type", Boolean)
], CustomSettings.prototype, "disableTriggers", void 0);
__decorate([
    class_validator_1.IsArray(),
    class_validator_1.ValidateNested({ each: true }),
    __metadata("design:type", Array)
], CustomSettings.prototype, "ignoredTables", void 0);
__decorate([
    class_validator_1.IsArray(),
    class_validator_1.ValidateNested({ each: true }),
    __metadata("design:type", Array)
], CustomSettings.prototype, "tablesToFill", void 0);
__decorate([
    class_validator_1.IsNumber(),
    class_validator_1.IsOptional(),
    __metadata("design:type", Number)
], CustomSettings.prototype, "maxLengthValue", void 0);
__decorate([
    class_validator_1.IsNumber(),
    class_validator_1.IsOptional(),
    __metadata("design:type", Number)
], CustomSettings.prototype, "seed", void 0);
__decorate([
    class_validator_1.IsNumber(),
    __metadata("design:type", Number)
], CustomSettings.prototype, "maxRowsPerBatch", void 0);
exports.CustomSettings = CustomSettings;
class CustomSchema {
    constructor() {
        this.settings = new CustomSettings();
        this.tables = [];
    }
    static fromJSON(json) {
        return __awaiter(this, void 0, void 0, function* () {
            const customSchema = class_transformer_1.plainToClass(CustomSchema, json);
            try {
                yield class_validator_1.validateOrReject(customSchema);
            }
            catch (errors) {
                throw new Error(errors + 'You should regenerate your schema.');
            }
            return customSchema;
        });
    }
    toJSON() {
        return class_transformer_1.classToPlain(this);
    }
}
__decorate([
    class_validator_1.ValidateNested(),
    __metadata("design:type", CustomSettings)
], CustomSchema.prototype, "settings", void 0);
__decorate([
    class_validator_1.IsArray(),
    class_validator_1.ValidateNested({ each: true }),
    class_transformer_1.Type(() => CustomTable),
    __metadata("design:type", Array)
], CustomSchema.prototype, "tables", void 0);
exports.CustomSchema = CustomSchema;
class CustomTable {
    constructor() {
        this.name = '';
        this.columns = [];
    }
}
__decorate([
    class_validator_1.IsString(),
    __metadata("design:type", String)
], CustomTable.prototype, "name", void 0);
__decorate([
    class_validator_1.IsArray(),
    class_validator_1.ValidateNested({ each: true }),
    __metadata("design:type", Array)
], CustomTable.prototype, "columns", void 0);
__decorate([
    class_validator_1.IsNumber(),
    __metadata("design:type", Number)
], CustomTable.prototype, "lines", void 0);
__decorate([
    class_validator_1.IsArray(),
    class_validator_1.ValidateNested({ each: true }),
    __metadata("design:type", Array)
], CustomTable.prototype, "before", void 0);
__decorate([
    class_validator_1.IsArray(),
    class_validator_1.ValidateNested({ each: true }),
    __metadata("design:type", Array)
], CustomTable.prototype, "after", void 0);
__decorate([
    class_validator_1.IsNumber(),
    __metadata("design:type", Number)
], CustomTable.prototype, "maxLines", void 0);
__decorate([
    class_validator_1.IsNumber(),
    __metadata("design:type", Number)
], CustomTable.prototype, "addLines", void 0);
__decorate([
    class_validator_1.IsBoolean(),
    __metadata("design:type", Boolean)
], CustomTable.prototype, "disableTriggers", void 0);
exports.CustomTable = CustomTable;
