"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Column = exports.Monotonic = exports.ForeignKey = exports.Table = exports.Schema = void 0;
const class_validator_1 = require("class-validator");
const class_transformer_1 = require("class-transformer");
const generators_1 = require("../generation/generators/generators");
class Schema {
    constructor() {
        this.tables = [];
    }
    static fromJSON(json) {
        return __awaiter(this, void 0, void 0, function* () {
            const schema = class_transformer_1.plainToClass(Schema, json);
            try {
                yield class_validator_1.validateOrReject(schema);
            }
            catch (errors) {
                throw new Error(errors + 'You should regenerate your schema.');
            }
            return schema;
        });
    }
    toJSON() {
        return class_transformer_1.classToPlain(this);
    }
}
__decorate([
    class_validator_1.IsArray(),
    class_validator_1.ValidateNested({ each: true }),
    class_transformer_1.Type(() => Table),
    __metadata("design:type", Array)
], Schema.prototype, "tables", void 0);
exports.Schema = Schema;
class Table {
    constructor() {
        this.name = '';
        this.columns = [];
        this.referencedTables = [];
    }
}
__decorate([
    class_validator_1.IsString(),
    __metadata("design:type", String)
], Table.prototype, "name", void 0);
__decorate([
    class_validator_1.IsArray(),
    class_validator_1.ValidateNested({ each: true }),
    class_transformer_1.Type(() => Column),
    __metadata("design:type", Array)
], Table.prototype, "columns", void 0);
__decorate([
    class_validator_1.IsArray(),
    __metadata("design:type", Array)
], Table.prototype, "referencedTables", void 0);
exports.Table = Table;
class ForeignKey {
    constructor() {
        this.table = '';
        this.column = '';
    }
}
__decorate([
    class_validator_1.IsString(),
    __metadata("design:type", String)
], ForeignKey.prototype, "table", void 0);
__decorate([
    class_validator_1.IsString(),
    __metadata("design:type", String)
], ForeignKey.prototype, "column", void 0);
exports.ForeignKey = ForeignKey;
var Monotonic;
(function (Monotonic) {
    Monotonic["ASC"] = "ASC";
    Monotonic["DESC"] = "DESC";
    Monotonic["NONE"] = "NONE";
})(Monotonic = exports.Monotonic || (exports.Monotonic = {}));
class Column {
    constructor() {
        this.name = '';
        this.generator = generators_1.Generators.none;
        this.nullable = 0;
        this.unique = false;
        this.autoIncrement = false;
        this.unsigned = false;
        this.min = 0;
        this.max = 255;
        this.monotonic = Monotonic.NONE;
    }
}
__decorate([
    class_validator_1.IsString(),
    __metadata("design:type", String)
], Column.prototype, "name", void 0);
__decorate([
    class_validator_1.IsString(),
    __metadata("design:type", String)
], Column.prototype, "generator", void 0);
__decorate([
    class_validator_1.IsNumber(),
    __metadata("design:type", Number)
], Column.prototype, "nullable", void 0);
__decorate([
    class_validator_1.IsBoolean(),
    __metadata("design:type", Boolean)
], Column.prototype, "unique", void 0);
__decorate([
    class_validator_1.IsBoolean(),
    __metadata("design:type", Boolean)
], Column.prototype, "autoIncrement", void 0);
__decorate([
    class_validator_1.IsBoolean(),
    __metadata("design:type", Boolean)
], Column.prototype, "unsigned", void 0);
__decorate([
    class_validator_1.IsNumber(),
    __metadata("design:type", Number)
], Column.prototype, "min", void 0);
__decorate([
    class_validator_1.IsNumber(),
    __metadata("design:type", Number)
], Column.prototype, "max", void 0);
__decorate([
    class_validator_1.IsString(),
    class_validator_1.IsOptional(),
    __metadata("design:type", String)
], Column.prototype, "minDate", void 0);
__decorate([
    class_validator_1.IsString(),
    class_validator_1.IsOptional(),
    __metadata("design:type", String)
], Column.prototype, "maxDate", void 0);
__decorate([
    class_validator_1.ValidateNested(),
    class_validator_1.IsOptional(),
    __metadata("design:type", ForeignKey)
], Column.prototype, "foreignKey", void 0);
__decorate([
    class_validator_1.ValidateNested(),
    class_validator_1.IsOptional(),
    __metadata("design:type", Object)
], Column.prototype, "values", void 0);
__decorate([
    class_validator_1.IsOptional(),
    class_validator_1.IsString(),
    __metadata("design:type", String)
], Column.prototype, "monotonic", void 0);
__decorate([
    class_validator_1.IsOptional(),
    class_validator_1.IsString(),
    __metadata("design:type", String)
], Column.prototype, "customFunction", void 0);
exports.Column = Column;
